/*
 * Created on Sep 13, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.agency;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.AgencyIntegrationSoapBindingStub;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.ApplicationInformationType;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListRequestApplicationFilter;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListRequestApplicationFilterFilter;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListResponse;

import gov.grants.business.util.CommonConstants;

import java.net.URL;
import java.rmi.RemoteException;
import java.security.Security;

import javax.xml.rpc.ServiceException;

import junit.framework.Assert;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

/**
 * @author Grants.gov WebServices Team
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class GetApplicationListTest
   extends AgencyWebServiceTestCase
{
   private static final Logger LOG = Logger.getLogger(GetApplicationListTest.class);
   
   public GetApplicationListTest(String test)
   {
      super(test);
   }
   
   public static void main(String[] args)
   {
      if (args.length > 0)
      {
         TestSuite suite = new TestSuite();
         for (int i = 0; i < args.length; i++)
         {
            LOG.info("Will run " + args[i]);
            suite.addTest(new GetApplicationListTest(args[i]));
         }
         TestRunner.run(suite);
      }
      else
      {
         LOG.info("Will run all tests.");
         TestRunner.run(GetApplicationListTest.class);
      }
   }
   
//   public void testGetApplicationListWithNullFilter()
//   {
//      LOG.debug("\n\n=======Start Get Application List With Null Filter =========");
//      
//      try
//      {
//    	  printList(getApplicationList(null, this));
//      }
//      catch (RemoteException e)
//      {
//         LOG.info("Expected exception: " + e.getMessage());
//      }
//      
//   }
   
   public void testGetApplicationListWithEmptyFilter()
   {
      LOG.debug("\n\n=======Start Get Application List With Empty Filter =========");
      GetApplicationListRequestApplicationFilter[] filter = new GetApplicationListRequestApplicationFilter[1];
      printList(getApplicationList(filter, this));
   }
   
   public void testGetApplicationListWithValidatedStatusFilter()
   {
      LOG.debug("\n\n=======Start Get Application List With Validated Status Filter ========");
      GetApplicationListRequestApplicationFilter[] filter = new GetApplicationListRequestApplicationFilter[1];
      GetApplicationListRequestApplicationFilterFilter af_filter = GetApplicationListRequestApplicationFilterFilter.fromString("Status");
      filter[0] = new GetApplicationListRequestApplicationFilter();
      filter[0].setFilter(af_filter);
//      filter[0].setFilterValue("Validated");
      filter[0].setFilterValue( CommonConstants.READY_FOR_DOWNLOAD );
      printList(getApplicationList(filter, this));
   }
   
   public void testGetApplicationListWithReceivedStatusFilter()
   {
      LOG.debug("\n\n=======Start Get Application List With Received Status Filter ========");
      GetApplicationListRequestApplicationFilter[] filter = new GetApplicationListRequestApplicationFilter[1];
      GetApplicationListRequestApplicationFilterFilter af_filter = GetApplicationListRequestApplicationFilterFilter.fromString("Status");
      filter[0] = new GetApplicationListRequestApplicationFilter();
      filter[0].setFilter(af_filter);
//      filter[0].setFilterValue("Received by Agency"); // value in old system
      filter[0].setFilterValue( CommonConstants.RECEIVED_BY_AGENCY );// value in new system
      printList(getApplicationList(filter, this));
   }
   
   public void testGetApplicationListWithBlankFilter()
   {
      LOG.debug("\n\n=======Start Get Application List With Blank Filter ========");
      GetApplicationListRequestApplicationFilter[] filter = new GetApplicationListRequestApplicationFilter[1];
      GetApplicationListRequestApplicationFilterFilter af_filter = GetApplicationListRequestApplicationFilterFilter.fromString("Status");
      filter[0] = new GetApplicationListRequestApplicationFilter();
      filter[0].setFilter(af_filter);
      filter[0].setFilterValue("");
      printList(getApplicationList(filter, this));
   }
   
   static String getExistingTrackingNumber(AgencyWebServiceTestCase test)
   {
      ApplicationInformationType[] list = null;
      GetApplicationListRequestApplicationFilter[] filter = new GetApplicationListRequestApplicationFilter[1];
      GetApplicationListRequestApplicationFilterFilter af_filter = GetApplicationListRequestApplicationFilterFilter.fromString("Status");
      filter[0] = new GetApplicationListRequestApplicationFilter();
      filter[0].setFilter(af_filter);
//      filter[0].setFilterValue("Validated");
      filter[0].setFilterValue( CommonConstants.READY_FOR_DOWNLOAD );
      list = GetApplicationListTest.getApplicationList(filter, test);
      if (list == null || list.length == 0)
      {
         Assert.fail("There are no applications available.");
      }
      return list[0].getGrants_govTrackingNumber();
   }
   
   static ApplicationInformationType[] getApplicationList(GetApplicationListRequestApplicationFilter[] request, AgencyWebServiceTestCase test)
   {   
      GetApplicationListResponse response = null;
      try
      {
         response = test.getPort().getApplicationList(request);
      }
      catch (RemoteException e)
      {
         LOG.fatal("Failed to invoke Get Application List call.", e);
         Assert.fail(e.getMessage());
      }
      catch (ServiceException e)
      {
         LOG.fatal("Unable to get port.", e);
         Assert.fail("Unable to get port.");
      }
      return response.getApplicationInformation();
   }
   
   public void printList(ApplicationInformationType[] list)
   {
      if (LOG.isDebugEnabled())
      {
         if (list == null)
         {
            LOG.debug("Application list is null.");
            return;
         }
         
         LOG.debug("Received " + list.length + " applications in list");
         
         StringBuffer buffer = new StringBuffer().append("\n");
         for (int i = 0; list != null && i < list.length; i++)
         {
            buffer.append("Application list element " + (i+1 < 10 ? "0" : "") + (i+1) + ": " + list[i].getGrants_govTrackingNumber() + "; Status: " + list[i].getGrants_GovApplicationStatus() + "\n");
         }
         LOG.debug(buffer.toString());
      }
   }

}
