/*
 * Created on Sep 13, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.agency;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.AgencyIntegrationPortType;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.AgencyIntegrationServicesLocator;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.AgencyIntegrationSoapBindingStub;
import gov.grants.apply.util.FileUtil;
import gov.grants.apply.util.PropertiesLoader;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.Security;
import java.util.StringTokenizer;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.rpc.ServiceException;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.SOAPException;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

/**
 * @author Grants.gov WebServices Team
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public abstract class AgencyWebServiceTestCase
   extends TestCase
{
   public static final String APPLICATION_PROPERTIES = "resources/application.properties";
   
   private static final String CID_PREFIX = "cid:";
   private static final int CID_PREFIX_LENGTH = CID_PREFIX.length();
   private static final String DEFAULT_ENDPOINT = "http://localhost:8080/agency-s2s-server/services/AgencyIntegrationSoapPort";
   private static final String AGENCY_ENDPOINT = "gov.grants.agency.endpoint";
   private static final String LOG4J_CONFIG = "log4j.configuration";
   private static final String TRUSTSTORE_PASSWORD = "javax.net.ssl.trustStorePassword";
   private static final String TRUSTSTORE = "javax.net.ssl.trustStore";
   private static final String KEYSTORE_PASSWORD = "javax.net.ssl.keyStorePassword";
   private static final String KEYSTORE = "javax.net.ssl.keyStore";
   private static final Logger log = Logger.getLogger( AgencyWebServiceTestCase.class.getName() );
   private static final URL ENDPOINT_URL;
   
   private AgencyIntegrationPortType _port = null;

   static
   {
      String config = PropertiesLoader.getValue(APPLICATION_PROPERTIES, LOG4J_CONFIG);
      if (config != null)
      {
         /*
          * LOG4J 1.2.8 refuses to automatically configure itself when the property
          * log4j.configuration is set to a URL where the file is not at the root level,
          * even though log4j's Loader is able to find the file.
          */
         
         System.setProperty(LOG4J_CONFIG, config); // if config doesn't have a '/' (i.e. the location is at the root of some classpath directory), then log4j can configure solely with this property set.

         int index = -1;
         if ((index = config.lastIndexOf('/')) > -1)
         {
            URL location = org.apache.log4j.helpers.Loader.getResource(config);
            if (location != null)
            {
               System.out.println("LOG4J configuration file = " + config + " = " + location);
               //System.setProperty(LOG4J_CONFIG, location.toString()); log4j won't configure with this.  Must do it manually.
               try
               {
                  DOMConfigurator.configure(location);
               }
               catch (FactoryConfigurationError e)
               {
                  e.printStackTrace();
                  BasicConfigurator.configure();
               }
            }
         }
      }
//      log = Logger.getLogger(AgencyWebServiceTestCase.class);
      
      URL endpoint = null;
      try
      {
         endpoint = PropertiesLoader.getURLValue(APPLICATION_PROPERTIES, AGENCY_ENDPOINT);
      }
      catch (MalformedURLException e)
      {
         String host = PropertiesLoader.getValue(APPLICATION_PROPERTIES, "SOAP_HOST");
         String port = PropertiesLoader.getValue(APPLICATION_PROPERTIES, "SOAP_PORT");
         try
         {
            endpoint = new URL(host+port);
         }
         catch (MalformedURLException e1)
         {
            try
            {
               endpoint = new URL(DEFAULT_ENDPOINT);
            }
            catch (MalformedURLException e2)
            {
               Assert.fail("Unable to create a SOAP port.");
            }
         }
      }
      ENDPOINT_URL = endpoint;
      if (log.isDebugEnabled())
      {
         log.debug("Endpoint: " + ENDPOINT_URL);
         log.debug("Endpoint protocol: " + ENDPOINT_URL.getProtocol());
      }

      Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());

   }
   
   /**
    * Constructor for AgencyWebServiceTestCase.
    * @param name
    */
   public AgencyWebServiceTestCase(String name)
   {
      super(name);
   }

   /*
    * @see TestCase#setUp()
    */
   protected void setUp() throws Exception
   {
      super.setUp();
      setSSLProperties();
   }
   
   private void setSSLProperties()
   {
      System.setProperty(KEYSTORE, PropertiesLoader.getValue(APPLICATION_PROPERTIES, KEYSTORE));
      System.setProperty(KEYSTORE_PASSWORD, PropertiesLoader.getValue(APPLICATION_PROPERTIES, KEYSTORE_PASSWORD));
      System.setProperty(TRUSTSTORE, PropertiesLoader.getValue(APPLICATION_PROPERTIES, TRUSTSTORE));
      System.setProperty(TRUSTSTORE_PASSWORD, PropertiesLoader.getValue(APPLICATION_PROPERTIES, TRUSTSTORE_PASSWORD));
   }
   
   protected AgencyIntegrationSoapBindingStub getPort()
      throws ServiceException
   {
      if (_port == null)
      {
         AgencyIntegrationServicesLocator locator = new AgencyIntegrationServicesLocator();

         _port = locator.getAgencyIntegrationSoapPort(ENDPOINT_URL);
         if (_port == null)
         {
            throw new ServiceException("Axis could not create a SOAP part.");
         }
      }
      return (AgencyIntegrationSoapBindingStub) _port;
   }
   
   protected void downloadAttachments(String prefix, String suffix)
   {
	   log.debug( "downloading attachments" );
      AgencyIntegrationSoapBindingStub stub = null;
      try {
         stub = getPort();
      } catch ( ServiceException e ) {
         log.fatal("Could not create SOAP port.", e);
         Assert.fail("Could not create SOAP port: " + e.getMessage());
      }// try-catch
      
      Object[] attachments = stub.getAttachments();
      log.debug( "attachments[]=" + attachments );
      if ( attachments == null ) {
    	  log.info( "no attachments returned" );
    	  return;
      }// if
      log.debug( "number of attachments=" + attachments.length );
      for ( int i = 0; i < attachments.length; i++ ) {
    	  
         AttachmentPart attachment = (AttachmentPart) attachments[i];
         if (attachment == null) {
            Assert.fail("AttachmentPart retrieved is null.");
         }// if
         
         String dir = PropertiesLoader.getValue(APPLICATION_PROPERTIES, "DIR_TO_WRITE_APPS");
         log.debug( "dir=" + dir );
//         String fileName = prefix + i + suffix;
         String fileName = prefix + "." + suffix;
         log.debug( "initial fileName=" + fileName );
         try {
            String contentId = attachment.getContentId();
            log.debug( "contentId=" + contentId );
            if ( contentId != null && !"".equals( contentId ) ) {
//               fileName = new StringTokenizer(contentId, "!@#$%^&*()+=\\/{}[]<> ").nextToken();
               fileName = new StringTokenizer(contentId, "!@#$%^&*()+=\\/{}[]<>").nextToken();
               log.debug( "'raw' fileName=" + fileName );
               int index = fileName.indexOf(CID_PREFIX);
               if ( index > -1 ) {
                  fileName = fileName.substring( index + CID_PREFIX_LENGTH );
               }// if
               log.debug( "'clean' fileName=" + fileName );
            }// if
            InputStream input = new BufferedInputStream( attachment.getDataHandler().getInputStream() );
            OutputStream output = new BufferedOutputStream( new FileOutputStream( FileUtil.formPath( dir, fileName ) ) );
            FileUtil.pipe( output, input );
            log.info( "Wrote attachment to " + FileUtil.formPath( dir, fileName ) );
            
         } catch (FileNotFoundException e) {
            log.fatal("Could not create the output zip file.", e);
            Assert.fail("Could not create the output zip file: " + e.getMessage());
         } catch (IOException e) {
            log.fatal("Could not open or read the zip file attachment.", e);
            Assert.fail("Could not read AttachmentPart: " + e.getMessage());
         } catch (SOAPException e) {
            log.fatal("Could not get the data handler from the attachment, there is no data in the AttachmentPart.", e);
            Assert.fail("AttachmentPart was empty: " + e.getMessage());
         }
      }
   }
   
    static {
	   // Initialize log4j
	   WSTestClient.init();
   }   
}
