/*
 * Created on Sep 14, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.agency;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.AgencyIntegrationSoapBindingStub;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationRequest;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationResponse;
import gov.grants.apply.util.FileUtil;
import gov.grants.apply.util.PropertiesLoader;

import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.rmi.RemoteException;

import javax.xml.rpc.ServiceException;

import junit.framework.Assert;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.apache.log4j.Logger;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;

/**
 * @author Grants.gov WebServices Team
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class GetApplicationTest
   extends AgencyWebServiceTestCase
{
   private static Logger LOG = Logger.getLogger(GetApplicationTest.class);
   
   private static String grantsGovTrackingNumber;

   /**
    * @param name
    */
   public GetApplicationTest(String name)
   {
      super(name);
   }

   public static void main(String[] args)
   {
      if (args.length > 0)
      {
   
		grantsGovTrackingNumber = args[0];
		LOG.debug("Will run getApplication() for " + grantsGovTrackingNumber);
    	   
		TestSuite suite = new TestSuite();
	    suite.addTest(new GetApplicationTest("testGetApplication"));
	    
//		for (int i = 0; i < args.length; i++)
//		{
//			LOG.info("Will run " + args[i]);
//		    suite.addTest(new GetApplicationTest("testGetApplication"));
//		}
		TestRunner.run(suite);
      }
      else
      {
         LOG.info("Will run all tests.");
         TestRunner.run(GetApplicationTest.class);
      }
   }
   
   public void testGetApplication()
   {
	  String trackingNum;
      if (grantsGovTrackingNumber != null) {
    	  trackingNum = grantsGovTrackingNumber;
      } else {
    	  trackingNum = GetApplicationListTest.getExistingTrackingNumber(this);
      }
      
      try
      {
         getApplication(trackingNum);
      }
      catch (RemoteException e)
      {
         LOG.fatal("Get application Web Service call failed.", e);
         Assert.fail("Get application Web Service call failed.");
      }
      
      // Reset
      grantsGovTrackingNumber = null;      
   }
   
   public void testGetApplicationWithBlankTrackingNum()
   {
      String trackingNum = "";
      try
      {
         getApplication(trackingNum);
      }
      catch (RemoteException e)
      {
         LOG.info("Expected exception: " + e.getMessage());
      }
   }
   
   public void testGetApplicationWithBadTrackingNum()
   {
      String trackingNum = "bad number";
      try
      {
         getApplication(trackingNum);
      }
      catch (RemoteException e)
      {
         LOG.info("Expected exception: " + e.getMessage());
      }
   }
   
   private void getApplication(String trackingNum)
      throws RemoteException
   {
       String dir = PropertiesLoader.getValue(AgencyWebServiceTestCase.APPLICATION_PROPERTIES, "DIR_TO_WRITE_APPS");
       LOG.error("********** dir: " +  dir + "\n\n\n");
      GetApplicationRequest request = new GetApplicationRequest();
      try
      {
         request.setEncodingType(WSTestClient.getAttachmentEncodingType()); //DIME or MIME
      }
      catch (Exception e)
      {
         LOG.debug("Could not set encoding type, using Axis default.", e);
      }
      request.setGrants_govTrackingNumber(trackingNum);
      GetApplicationResponse response = null;
      AgencyIntegrationSoapBindingStub stub = null;
      try
      {
         stub = getPort();
      }
      catch (ServiceException e)
      {
         LOG.fatal("Could not create SOAP port.", e);
      }
      response = stub.getApplication(request);
      
      if (response != null)
      {
         Document grantApp = (Document) response.getSubmittedApplication();
         dir = PropertiesLoader.getValue(AgencyWebServiceTestCase.APPLICATION_PROPERTIES, "DIR_TO_WRITE_APPS");
         LOG.debug("********** dir: " +  dir + "\n\n\n");
         if (grantApp != null)
         {
            try
            {
               String applicationFile = FileUtil.formPath(dir, trackingNum + ".xml");
               OutputFormat format = new OutputFormat(grantApp);
               format.setPreserveSpace(true);
               OutputStream output = new BufferedOutputStream(new FileOutputStream(applicationFile));
               XMLSerializer serializer = new XMLSerializer(output, format);
               serializer.serialize(grantApp);
               if (LOG.isInfoEnabled())
               {
                  LOG.info("Wrote application to " + applicationFile);
               }
            }
            catch (FileNotFoundException e)
            {
               LOG.fatal("Could not open file to write.", e);
               Assert.fail("Could not open file to write: " + e.getMessage());
            }
            catch (IOException e)
            {
               LOG.fatal("Could not write application XML to file.", e);
               Assert.fail("Could not write application XML to file: " + e.getMessage());
            }
         }
         else
         {
            Assert.fail("GetApplication did not return an application that should be in the system.");
         }
         downloadAttachments(trackingNum, "");
      }
   }
  
}
