/*
 * Created on Sep 13, 2005
 *
 * Unpublished Work, Copyright, 2005 Northrop Grumman
 */
package gov.grants.apply.agency;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.EncodingTypeEnum;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationZipRequest;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationZipResponse;

import java.rmi.RemoteException;

import javax.xml.rpc.ServiceException;

import junit.framework.Assert;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.apache.log4j.Logger;

/**
 * @author Grants.gov WebServices Team
 */
public class GetApplicationZipTest
   extends AgencyWebServiceTestCase
{
   private static Logger log = Logger.getLogger(GetApplicationZipTest.class); 
   
   private static String grantsGovTrackingNumber;   
   
   /**
    * @param name
    */
   public GetApplicationZipTest(String name)
   {
      super(name);
   }
   
   public static void main(String[] args)
   {
      if (args.length > 0)
      {
  		grantsGovTrackingNumber = args[0];
		log.debug("Will run getApplicationZip() for " + grantsGovTrackingNumber);
    	   
		TestSuite suite = new TestSuite();
	    suite.addTest(new GetApplicationZipTest("testGetApplicationZip"));
//         for (int i = 0; i < args.length; i++)
//         {
//            log.info("Will run " + args[i]);
//            suite.addTest(new GetApplicationZipTest(args[i]));
//         }
         TestRunner.run(suite);
      }
      else
      {
         log.info("Will run all tests.");
         TestRunner.run(GetApplicationZipTest.class);
      }
   }
   
   public void testGetApplicationZip()
   {
	  String trackingNum;
      if (grantsGovTrackingNumber != null) {
    	  trackingNum = grantsGovTrackingNumber;
      } else {
    	  trackingNum = GetApplicationListTest.getExistingTrackingNumber(this);
      }
      
      if (trackingNum == null || "".equals(trackingNum))
      {
         Assert.fail("Application list returned an application with a null or blank tracking number.");
      }
      try
      {
         getApplicationZip(trackingNum);
      }
      catch (RemoteException e)
      {
         log.fatal("The GetApplicationZip web service call failed.", e);
         Assert.fail("The GetApplicationZip web service call failed: " + e.getMessage());
      }
      
      // Reset
      grantsGovTrackingNumber = null;
   }
   
   public void testGetAppZipWithBadTrackingNum()
   {
      String trackingNumber = new String("bad number");
      try
      {
         getApplicationZip(trackingNumber);
      }
      catch (RemoteException e)
      {
         // We expect this with a "bad number"
         log.info("Expected expection with this test: " + e.getMessage());
      }
   }
   
   public void testGetAppZipWithEmptyTrackingNum()
   {
      String trackingNumber = new String("");
      try
      {
         getApplicationZip(trackingNumber);
      }
      catch (RemoteException e)
      {
         // We expect this with ""
         log.info("Expected expection with this test: " + e.getMessage());
      }
   }
   
   private void getApplicationZip(String trackingNumber)
      throws RemoteException
   {
      GetApplicationZipRequest request = new GetApplicationZipRequest();
      try
      {
         EncodingTypeEnum type = WSTestClient.getAttachmentEncodingType();
         request.setEncodingType(type); //DIME or MIME
      }
      catch (Exception e)
      {
         log.fatal("Could not get the attachment encoding type property, so just using Axis default.", e);
      }
      
      request.setGrants_govTrackingNumber(trackingNumber);
      GetApplicationZipResponse response = null;
      try
      {
         response = getPort().getApplicationZip(request);
      }
      catch (ServiceException e)
      {
         log.fatal("Could not get SOAP port", e);
         Assert.fail("Could not get SOAP port: " + e.getMessage());
      }
      log.debug( "downloading attachments" );
      downloadAttachments(trackingNumber, "zip");
   }
   
}
