/*
 * Created on July 9, 2008
 *
 * Copyright, 2008 General Dynamics IT
 */
package gov.grants.apply.agency;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.SaveAdobeOpportunityRequest;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.SaveAdobeOpportunityResponse;
import gov.grants.business.util.FileUtil;

import java.rmi.RemoteException;

import javax.xml.rpc.ServiceException;

import junit.framework.Assert;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.apache.log4j.Logger;

/**
 * @author Grants.gov WebServices Team
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class SaveAdobeOpportunityTest extends AgencyWebServiceTestCase {
	private static final Logger log = Logger.getLogger( SaveAdobeOpportunityTest.class );
   
	private static String OPP_XML = null;
   
   /**
    * @param name
    */
	public SaveAdobeOpportunityTest( String name ) {
		super( name );
	}

	public static void main( String[] args ) {
		if ( args.length > 0 ) {
			String oppXmlPath = args[0];
			log.debug( "Loading Opportunity XML from:" + oppXmlPath );
			
			try {
				byte[] fileBytes = FileUtil.readFile( oppXmlPath );
				OPP_XML = new String( fileBytes );
				log.debug( "--- oppXml=" + OPP_XML );
			} catch ( Exception e ) {
				log.error( "Exception caught reading file [" + oppXmlPath + "]:" + e.getMessage() );
			}// try-catch
			
			if ( OPP_XML == null) {
				Assert.fail( "Could not load opportunity XML:" + oppXmlPath );
			}// if
			
			TestSuite suite = new TestSuite();
		    suite.addTest( new SaveAdobeOpportunityTest( "testSaveAdobeOpportunity" ) );    	  
		    TestRunner.run( suite );
		} else {
			log.info( "Path to opportunity XML required" );
			Assert.fail( "Path to opportunity XML required" );
		}// if-else
	}// main()
   
	public void testSaveAdobeOpportunity() {
		try {
			saveAdobeOpportunity( OPP_XML );
		} catch (RemoteException e) {
			log.fatal( "SaveAdobeOpportunity web service call failed:", e );
			Assert.fail( "SaveAdobeOpportunity web service call failed: " + e.getMessage() );
		}// try-catch
      
		// Reset
		OPP_XML = null;
	}// testSaveAdobeOpportunity()
	
	
	private void saveAdobeOpportunity( String oppXml )
	throws RemoteException {
		SaveAdobeOpportunityRequest request = new SaveAdobeOpportunityRequest();
		request.setOpportunityXML( oppXml );
		SaveAdobeOpportunityResponse response = null;
		try {
			log.debug( "Send opportunity XML" );
			response = getPort().saveAdobeOpportunity( request );
			log.debug( "--- response=" + response );
			log.debug( "--- opportunity number=" + response.getOpportunityNumber() );
			log.debug( "--- response message=" + response.getResponseMessage() );
		} catch ( ServiceException e ) {
			log.fatal( "Could not get SOAP port:", e );
			Assert.fail( "Could not get SOAP port: " + e.getMessage() );
		} catch ( Exception e ) {
			log.fatal( "Exception caught:" + e.getMessage() );
			Assert.fail( "Exception caught:" + e.getMessage() );
		}// try-catch
		
	}// saveAdobeOpportunity()
   
}
