/*
 * Created on Feb 17, 2006
 *
 * Unpublished Work, Copyright, 2005, 2006 Northrop Grumman
 */
package gov.grants.common.sql;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * @author david.wong2@ngc.com, Grants.gov Team 
 *
 * TODO Add description.
 */
public final class SelectBuilder implements SQLString
{
   private ArrayList _tables = null;
   private ArrayList _columns = null;
   private ConditionsGroup _where = null;
   private OrderBy _orderby = null;
   
   public String toString()
   {
      StringBuffer buffer = new StringBuffer();
      addString(buffer);
      return buffer.toString();
   }
   
   public StringBuffer addString(StringBuffer buffer)
   {
      if (_tables == null || _columns == null)
      {
         return buffer;
      }
      
      buffer.append("select ");
      Iterator iter = _columns.iterator();
      while (iter.hasNext())
      {
         ((Table.Column) iter.next()).addString(buffer);
         if (iter.hasNext())
         {
            buffer.append(", ");
         }
      }
      buffer.append(" from ");
      iter = _tables.iterator();
      while (iter.hasNext())
      {
         ((Table) iter.next()).addString(buffer);
         if (iter.hasNext())
         {
            buffer.append(", ");
         }
      }
      
      if (_where != null)
      {
         _where.addString(buffer.append(" where "));
      }
      
      if (_orderby != null)
      {
         _orderby.addString(buffer.append(" "));
      }
      
      return buffer;
   }
   
   public Table addTable(String name)
   {
      Table table = new Table(name);
      if (_tables == null)
      {
         _tables = new ArrayList();
      }
      _tables.add(table);
      return table;
   }
   
   public void addColumn(Table.Column column)
   {
      if (_columns == null)
      {
         _columns = new ArrayList();
      }
      _columns.add(column);
   }
   
   public void addOrderBy(SQLString operand)
   {
      if (_orderby == null)
      {
         _orderby = new OrderBy(operand);
      }
      else
      {
         _orderby.add(operand);
      }
   }
   
   public ConditionsGroup getWhereConditions()
   {
      if (_where == null)
      {
         _where = new ConditionsGroup();
      }
      return _where;
   }
   
   public static void main(String[] args)
   {
      SelectBuilder select = new SelectBuilder();
      Table tstatus = select.addTable("tstatus");
      Table tformdata = select.addTable("tformdata");
      select.addColumn(tformdata.column("fundoppnumber"));
      select.addColumn(tstatus.column("processid"));
      select.addColumn(tstatus.column("status"));
      select.getWhereConditions().andEquals(tformdata.column("processid"), tstatus.column("processid"));
      select.getWhereConditions().andNotEquals(tstatus.column("status"), new TextLiteral("Purged"));
      select.getWhereConditions().andEquals(tformdata.column("agencyid"), new TextLiteral("HUD"));
      Group group = select.getWhereConditions().andGroup();
      group.andLessThan(new Function("length", tformdata.column("fundoppnumber")), new Literal("1"));
      group.orGreaterThan(new Function("length", tformdata.column("fundoppnumber")), new Literal("100"));
      select.addOrderBy(tformdata.column("fundoppnumber"));
      select.addOrderBy(tstatus.column("processid"));
      ArrayList cfdaList = new ArrayList();
      cfdaList.add("00.000");
      cfdaList.add("11.001");
      cfdaList.add("98.001");
      Group cfdaGroup = select.getWhereConditions().andGroup();
      cfdaGroup.orEquals(tformdata.column("cfda"), cfdaList, new TextLiteral(""));
      
      ArrayList isNull = new ArrayList();
      isNull.add(tformdata.column("fundoppnumber"));
      isNull.add(tstatus.column("processid"));
      Group isNullGroup = select.getWhereConditions().andGroup();
      isNullGroup.orIsNull(isNull);
      isNullGroup.andIsNotNull(isNull);
      
      System.out.println(select);
   }
}
